---------------------------------------------------------------------------------															          
---------------------------------------------------------------------------------
------------------ OUTPUT Params ------------------------------------------
---------------------------------------------------------------------------------
---------------------------------------------------------------------------------
USE SPWorkshop;
GO

ALTER PROCEDURE dbo.OutputParams
(
@ID INT,
@MyDB NVARCHAR(100) OUTPUT
)

AS

/*
This SP shows a simple output param.
Output params can be very useful to avoid
extra coding by jumping through hoops.
They let you assign vars directly from 
SP calls.

Sample Call:
DECLARE @NewDB nvarchar(100);
EXEC dbo.OutputParams @ID = 1, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
*/

IF @ID = 1
	BEGIN
		SET @MyDB = 'DB1';
	END
IF @ID = 2
	BEGIN
		SET @MyDB = 'DB2';
	END

GO

DECLARE @NewDB nvarchar(100);
EXEC dbo.OutputParams @ID = 1, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
/*
Let's slow down and take a look at the call again.
It does something weird.
The call is backwards from what you'd think it is,
but something had to win
in this dichotomy.

1. Here's how you assign a value to a var.
DECLARE @a int;
SET @a = 1;

2. Here's how you call an SP and pass-in a value from a param.
DECLARE @a int;
SET @a = 1;
EXEC dbo.SP1 @ID = @a;
EXEC dbo.SP1 @ID = 1;
So when the var is being assigned a value, it's on the left.
And when it's being used as the value, it's on the right.

Now how do you handle it when you need to both pass-in a
var as a value AND assign it a value at the same time?

In this case, #2 wins. We're assigning a value to the var
with the var on the right side.
*/

----Now let's take a look at it again.
----Here we're not passing anything in through @NewDB.
GO
DECLARE @NewDB nvarchar(100);
EXEC dbo.OutputParams @ID = 2, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
----So let's assign it a value. We'll have to change the
----SP to take advantage of it first.
GO

ALTER PROCEDURE dbo.OutputParams
(
@ID INT,
@MyDB NVARCHAR(100) OUTPUT
)

AS

/*
This SP shows a simple output param.
Output params can be very useful to avoid
extra coding by jumping through hoops.
They let you assign vars directly from 
SP calls.

Sample Call:
DECLARE @NewDB nvarchar(100);
EXEC dbo.OutputParams @ID = 1, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
*/

IF @ID = 1 AND @MyDB = 'xxx'
	BEGIN
		SET @MyDB = 'DB1';
	END
IF @ID = 2 AND @MyDB = 'yyy'
	BEGIN
		SET @MyDB = 'DB2';
	END
GO

DECLARE @NewDB nvarchar(100) = 'yyy';
EXEC dbo.OutputParams @ID = 2, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
----So do you have to pass it in as a
----named param? No.
GO
DECLARE @NewDB nvarchar(100) = 'yyy';
EXEC dbo.OutputParams 2, @NewDB OUTPUT;
SELECT @NewDB;
----But can you assign a default value
----to an output param? Y/N.
----We'll start by altering the SP again.
GO

ALTER PROCEDURE dbo.OutputParams
(
@ID INT,
@MyDB NVARCHAR(100) = 'yyy' OUTPUT
)

AS

/*
This SP shows a simple output param.
Output params can be very useful to avoid
extra coding by jumping through hoops.
They let you assign vars directly from 
SP calls.

Sample Call:
DECLARE @NewDB nvarchar(100);
EXEC dbo.OutputParams @ID = 1, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
*/

IF @ID = 1 AND @MyDB = 'xxx'
	BEGIN
		SET @MyDB = 'DB1';
	END
IF @ID = 2 AND @MyDB = 'yyy'
	BEGIN
		SET @MyDB = 'DB2';
	END
GO
--Don't assign @NewDB.
DECLARE @NewDB nvarchar(100);
EXEC dbo.OutputParams @ID = 2, @MyDB = @NewDB OUTPUT;
SELECT @NewDB;
----We've given it a default value, so
----why didn't it work?
----Cause it does get assigned when we
----make the call. It gets assigned the
----NULL that @NewDB has from not being
----assigned.

----So is there a way to 'activate' the
----default param? Again, Y/N.
----Since a default means you don't have
----to pass the param in. That will
----activate the default value. But
----w/o passing it in, there's no way
----to get the value back out. So y you
----can assign the default, but you 
----can't use it.





